/*
 *		 MALFORM.C (was MFORMAT.C)  MS Johnson  26-06-1990
 *
 *
 *			     Copyright (c) 1990
 *
 *		Copyright (c) 1990 KRAM Enterprises, USA
 *		Copyright (c) 1990 KRAMIT Limited, UK
 *		Copyright (c) 1990 KRAMSHIT, Slovenia
 *		Copyright (c) 1990 KRAMSKU, Finlandia
 *
 *
 *	MAY BE FREELY DISTRIBUTED WITH THE PERMISSION OF THE AUTHOR
 *
 *	As with most things in life, use is at your own risk (except
 *	in the States where accidents and misfortunes are not permitted
 *	under the lawyers' Bill of Free Enterprise).
 *
 *	Mark S. Johnson, Modelling and Biocomputing Group, Center
 *	for Biotechnology, PO Box 123, FIN-20521 TURKU, FINLAND
 *	FAX: 358-21-633-8000 Email: mark@hops.btk.utu.fi
 *
 *	multiple alignment program developed by the author at home
 *	and with his own resources.
 *
 *	reformating program
 *
 */

/*
 *	Modification history:

 *	1.0 -- original version. MSJ
 *	1.1 -- corrected for codes missing ">P1;". MSJ 9-8-90
 *	2.0 -- changed atoi usage per CONVEX; msj 5-3-91
 *	2.1 -- changes for SPROF and plain PIR files; msj 18-4-91
 *	2.2 -- changes to header info from malign and sprof; msj 29-6-91
 *	2.3 -- added percent identity calculations;  msj 4-7-91 
 *	2.4 -- added conservation plot to output; msj 6-3-92
 *	2.5 -- added scoring matrices; msj 19-4-93
 *	3.0 -- added -kit for kitsch input; -pca for principle comp file (PCA.LOCAL)
 *	4.0 -- name change to avoid conflicts with unix dos prog
 *	4.0.1- # at column 0 ignored at beginning of files msj 4-1-95
 */

#include <stdio.h>
#include <ctype.h>
#include "malform.h"
 
#define MAT_VER	"[matrix version 10.0 (last update: 21-4-94)]"
#define DEF_WID	65			/* print width of alignment */
#define DEF_MAT	0			/* default matrix: birkbeck*/
#define DEF_PEN	40			/* default non-negative gap penalty */
#define SEG	7

extern int ind[MAX_IND];

void malread();
void malhigh();
void malread();
void m_score();

main(ac,av)
int ac;
char *av[];
{
register i,j,k,c;
int width;
int k_save;
int len;
int a_flag;
int p_flag;
int c_flag;
int m_flag;
int r_flag;
int set;
int ctt;
char cc;
int ns;			/* number of sequences */
int pen;		/* pen value to use */
int mat;		/* mat number to use */
int kitsch;		/* kitsch input file if ==1 */
int pca;		/* PCA input file if ==1 */
int out;
int score;		/* alignemnt score */
int t_len;		/* length of alignment */
float tscore;		/* sum of normalized scores */

extern char m_names[MAX_MAT][MAX_CDE];	/* matrix names from matxx.h */

char Temp[MAX_CDE];		/* used to read args */

width=DEF_WID;

a_flag=0;
p_flag=0;
c_flag=0;
m_flag=0;

/*
 *	enter matrix number; penalty
 */

mat=DEF_MAT;
pen=DEF_PEN;
kitsch=0;
pca=0;
out=0;

set=1;

while(set<ac)
	{
	if(strncmp(av[set],"-kit",4)==0)
		{
		out=1;
		kitsch=1;
		m_flag=1;
		set++;
		}
	else if(strncmp(av[set],"-pca",4)==0)
		{
		out=1;
		pca=1;
		m_flag=1;
		set++;
		}
	else if(strncmp(av[set],"-A",2)==0)
		{
		a_flag=1;
		set++;
		}
	else if(strncmp(av[set],"-H",2)==0)
		{
	fprintf(stderr,"\n\nProgram to format sequence alignments");
	fprintf(stderr,"\n\nUsage: %s [-H] [-kit] [-pca] [-A] [-I] [-C] [-W#] [-M#] [-P#] < pir_file ",
		av[0]);
	fprintf(stderr,"\n\nFile_of_seqs -- sequences in PIR format!");
	fprintf(stderr,"[>output]",av[0]);
	fprintf(stderr,"\n-H : help message");
	fprintf(stderr,"\n-kit : Kitsch input file only (default matrix, penalty)");
	fprintf(stderr,"\n-pca : PCA (LOCAL) file only (default matrix, penalty)");
	fprintf(stderr,"\n-A : will output all subalignments (malign input)");
	fprintf(stderr,"\n-I : will compute percent identities for sequences");
	fprintf(stderr,"\n-C : turn on conservation plot; average over 5 residues");
	fprintf(stderr,"\n-W# : width of alignment, default = %d",DEF_WID);
	fprintf(stderr,"\n-M# : scoring matrix, # = an integer");
	fprintf(stderr,"; -P# : gap penalty, # = an integer");
	fprintf(stderr,"\nMatrix vals:    %s= 0  pen = 40; (DEFAULT)\n",m_names[0]);
	for(i=1;i<MAX_MAT;i++)
		{
		if(i%2!=0)fprintf(stderr,"%-20.20s	= %d",m_names[i],i);
		else fprintf(stderr,"		%-20.20s	= %d\n",m_names[i],i);
		}
	fprintf(stderr,"\n%s\n",VER);
	fprintf(stderr,"%s\n",MAT_VER);
	exit(-1);
		set++;
		}
	else if(strncmp(av[set],"-I",2)==0)
		{
		p_flag=1;
		set++;
		}
	else if(strncmp(av[set],"-C",2)==0)
		{
		c_flag=1;
		set++;
		}
	else if(strncmp(av[set],"-W",2)==0)
		{
		i=2;
		while(av[set][i]!='\0')
			{
			Temp[i-2]=av[set][i];
			i++;
			}
		Temp[i-2]='\0';
		width=atoi(Temp);
		set++;
		}
	else if(strncmp(av[set],"-M",2)==0)
		{
		i=2;
		m_flag=1;
		while(av[set][i]!='\0')
			{
			Temp[i-2]=av[set][i];
			i++;
			}
		Temp[i-2]='\0';
		mat=atoi(Temp);
		set++;
		}
	else if(strncmp(av[set],"-P",2)==0)
		{
		i=2;
		m_flag=1;
		while(av[set][i]!='\0')
			{
			Temp[i-2]=av[set][i];
			i++;
			}
		Temp[i-2]='\0';
		pen=atoi(Temp);
		set++;
		}
	else	{
	fprintf(stderr,"\nUnknown option: %s\n",av[set]);
	fprintf(stderr,"\n\nUsage: %s [-H] [-kit] [-pca] [-A] [-I] [-C] [-W#] [-M#] [-P#] < pir_file ",
		av[0]);
	fprintf(stderr,"\n\nFile_of_seqs -- sequences in PIR format!");
	fprintf(stderr,"[>output]",av[0]);
	fprintf(stderr,"\n-H : help message");
	fprintf(stderr,"\n-kit : Kitsch input file only (default matrix, penalty)");
	fprintf(stderr,"\n-pca : PCA (LOCAL) file only (default matrix, penalty)");
	fprintf(stderr,"\n-A : will output all subalignments (malign input)");
	fprintf(stderr,"\n-I : will compute percent identities for sequences");
	fprintf(stderr,"\n-C : turn on conservation plot; average over 5 residues");
	fprintf(stderr,"\n-W# : width of alignment, default = %d",DEF_WID);
	fprintf(stderr,"\n-M# : scoring matrix, # = an integer");
	fprintf(stderr,"; -P# : gap penalty, # = an integer");
	fprintf(stderr,"\nMatrix vals:    %s= 0  pen = 40; (DEFAULT)\n",m_names[0]);
	for(i=1;i<MAX_MAT;i++)
		{
		if(i%2!=0)fprintf(stderr,"%-20.20s	= %d",m_names[i],i);
		else fprintf(stderr,"		%-20.20s	= %d\n",m_names[i],i);
		}
	fprintf(stderr,"\n%s\n",VER);
	fprintf(stderr,"%s\n",MAT_VER);
		exit(49);
		}
	}

if(mat>=MAX_MAT)
	{
	fprintf(stderr,"\n\nMAX_MAT out of range\n\n");
	exit(-133);
	}
else if(out==1);
else	{
	printf(".nf\n\\f(CR");
	printf("\n\n***********************************");
	printf("****************************************");
	printf("\n*************************************");
	printf("**************************************");
	printf("\n*****  %s: %s ",av[0],VER);
	printf("\n*************************************");
	printf("**************************************");
	printf("\n*************************************");
	printf("**************************************");
	}

/* score alignments if m_flag==1 */

if(m_flag==1 && out!=1)
	{
	printf("\n");
	printf("\nMatrix:  %s",m_names[mat]);
	printf("    Penalty: %d",pen);

	printf("\n\n************************************");
	printf("***************************************\n\n");

	}

/*
 *	grab matrix and penalty and other comments
 */

r_flag=0;

for(;;) {
	fscanf(stdin," %c",&cc);
	if(cc=='#')
		{
		while((getchar())!='\n');
		r_flag=3;
		a_flag=3;
		}
	else if(cc=='*')
		{
		while((getchar())=='*')while((getchar())!='\n');
		if(out!=1)printf("\n\n");
		while((c=getchar())!='*' && out!=1)putchar(c);
		}
	else if(cc=='>')
		{
		ungetc(cc,stdin);
		r_flag=0;
		a_flag=3;
		}
	else	{
		if(r_flag!=3)rewind(stdin);
		else 	{
			r_flag=2;
	/*		ungetc(cc,stdin);*/
			}
		}
	if(r_flag<3)break;
	}
	
/*
 *	grab data from standard input
 */

for(;;)	{
	if(a_flag==0)
		{
		while((c=getchar())!='^' && c!=EOF);
		while((c=getchar())!='\n' && c!=EOF);
		}
	else if(a_flag==1)
		{
		while(((c=getchar())!='^'  && c!='~') && c!=EOF);
		while((c=getchar())!='\n' && c!=EOF);
		}
	else;
	if(c==EOF)
		{
		break;
		}

	/*
	 *	read sequences	
	 */

	malread(a_flag, &ns, &len);

	/*
	 *	highlight sequences
	 */

	if(c_flag==1) malhigh(ns, len);

	/*
	 *	now print it out
	 */

	if(out!=1)
		{
		malprint(c_flag, ns, len, width);
		}

	/*
	 *	percentage identities
	 */

	if(p_flag==1)
		{
		malpid(ns, len);
		}

/*
	j=0;
	len=0;
	ns=0;
	for(;;)	{
		if((c=getchar())==EOF)break;
		else if(a_flag==0 && c=='^')break;
		else if(a_flag==1 && (c=='^' || c=='~'))break;
		else if(c=='\n');
		else ungetc(c,stdin);

*/
	/* if m_flag==1, print alignment scores and normalized values */

	if(m_flag==1)m_score(ns,mat,pen,len,out,kitsch,pca);

	if(out!=1)
		{
		printf("\n***********************************");
		printf("****************************************");
		if(a_flag==1);
		else break;
		}
	}
if(out!=1)printf("\n");
if(out!=1)while((c=getchar())!=EOF)putchar(c);
	
exit(0);
}
